@@ -3,22 +3,32 @@
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
+import logging
 import sys
 from functools import wraps
 from pathlib import Path
-from typing import Any, Callable, List
+from typing import Any, Callable, List, Sequence
 
 import click
 from moreorless.click import echo_color_unified_diff
 
+from usort.translate import render_node
+
 from . import __version__
+from .api import usort_path, usort_stdin
 from .config import Config
-from .sorting import sortable_blocks, usort_path, usort_stdin
-from .util import TIMINGS, print_timings, try_parse
+from .sorting import ImportSorter
+from .types import Options
+from .util import get_timings, print_timings, Timing, try_parse
 
 BENCHMARK = False
 
 
+def print_benchmark(timings: Sequence[Timing]) -> None:
+    if BENCHMARK:
+        print_timings(click.echo, timings=timings)
+
+
 def usort_command(fn: Callable[..., int]) -> Callable[..., None]:
     """
     Run wrapped command, print timings if --benchmark, and exit with return code
@@ -26,22 +36,28 @@ def usort_command(fn: Callable[..., int]) -> Callable[..., None]:
 
     @wraps(fn)
     def wrapper(*args: Any, **kwargs: Any) -> None:
-        TIMINGS.clear()
         exit_code = fn(*args, **kwargs) or 0
         if BENCHMARK:
-            print_timings(click.echo)
+            print_timings(click.echo, timings=get_timings())
         sys.exit(exit_code)
 
     return wrapper
 
 
 @click.group()
+@click.pass_context
 @click.version_option(__version__, "--version", "-V")
+@click.option("--debug", is_flag=True, help="Enable debug output")
 @click.option("--benchmark", is_flag=True, help="Output benchmark timing info")
-def main(benchmark: bool) -> None:
+def main(ctx: click.Context, benchmark: bool, debug: bool) -> None:
     global BENCHMARK
     BENCHMARK = benchmark
 
+    level = logging.DEBUG if debug else logging.WARNING
+    logging.basicConfig(level=level, stream=sys.stderr)
+
+    ctx.obj = Options(debug=debug)
+
 
 @main.command()
 @click.option("--multiples", is_flag=True, help="Only show files with multiple blocks")
@@ -56,10 +72,12 @@ def list_imports(multiples: bool, debug: bool, filenames: List[str]) -> int:
     # where the barriers are that produce different blocks.
 
     for f in filenames:
-        config = Config.find(Path(f))
-        mod = try_parse(Path(f))
+        path = Path(f)
+        config = Config.find(path)
+        mod = try_parse(path)
         try:
-            blocks = sortable_blocks(mod.body, config)
+            sorter = ImportSorter(module=mod, path=path, config=config)
+            blocks = sorter.sortable_blocks(mod.body)
         except Exception as e:
             print("Exception", f, e)
             continue
@@ -70,18 +88,19 @@ def list_imports(multiples: bool, debug: bool, filenames: List[str]) -> int:
         click.secho(f"{f} {len(blocks)} blocks:", fg="yellow")
         for b in blocks:
             print(f"  body[{b.start_idx}:{b.end_idx}]")
-            sorted_stmts = sorted(b.stmts)
+            sorted_imports = sorted(b.imports)
             if debug:
-                for s in b.stmts:
+                for imp in b.imports:
                     print(
-                        f"    {sorted_stmts.index(s)} {s} "
-                        f"({s.config.category(s.first_module)})"
+                        f"    {sorted_imports.index(imp)} {imp} "
+                        f"({imp.config.category(imp.stem or '')})"
                     )
             else:
                 print("Formatted:")
                 print("[[[")
-                for s in sorted_stmts:
-                    print(mod.code_for_node(s.node), end="")
+                for imp in sorted_imports:
+                    assert imp.node is not None
+                    print(render_node(imp.node), end="")
                 print("]]]")
 
     return 0
@@ -105,17 +124,23 @@ def check(filenames: List[str]) -> int:
                 click.echo(f"Error sorting {result.path}: {result.error}")
                 return_code |= 1
 
+            for warning in result.warnings:
+                click.echo(f"Warning at {result.path}:{warning.line} {warning.message}")
+
             if result.content != result.output:
                 click.echo(f"Would sort {result.path}")
                 return_code |= 2
 
+            print_benchmark(result.timings)
+
     return return_code
 
 
 @main.command()
+@click.pass_context
 @click.argument("filenames", nargs=-1)
 @usort_command
-def diff(filenames: List[str]) -> int:
+def diff(ctx: click.Context, filenames: List[str]) -> int:
     """
     Output diff of changes for one or more path
     """
@@ -128,7 +153,13 @@ def diff(filenames: List[str]) -> int:
         for result in usort_path(path, write=False):
             if result.error:
                 click.echo(f"Error sorting {result.path}: {result.error}")
+                if ctx.obj.debug:
+                    click.echo(result.trace)
                 return_code |= 1
+                continue
+
+            for warning in result.warnings:
+                click.echo(f"Warning at {result.path}:{warning.line} {warning.message}")
 
             if result.content != result.output:
                 assert result.encoding is not None
@@ -138,6 +169,8 @@ def diff(filenames: List[str]) -> int:
                     result.path.as_posix(),
                 )
 
+            print_benchmark(result.timings)
+
     return return_code
 
 
@@ -168,9 +201,14 @@ def format(filenames: List[str]) -> int:
                 return_code |= 1
                 continue
 
+            for warning in result.warnings:
+                click.echo(f"Warning at {result.path}:{warning.line} {warning.message}")
+
             if result.content != result.output:
                 click.echo(f"Sorted {result.path}")
 
+            print_benchmark(result.timings)
+
     return return_code
 
 
